# include	<CListenerThread.h>
# include	<ErrorUtilities.h>


CListenerThread::CListenerThread( const char* inName,
	ulong inInterface, int inPort )
:	CThread( inName, B_NORMAL_PRIORITY )
{
	mListener = new CTCPStream( inName );
	if ( !mListener )
		/* ERROR */;
		
	mInterface = inInterface;
	mPort = inPort;
}

CListenerThread::~CListenerThread( )
{

}


//=========================================================================
//	* CListenerThread::RunThread
//=========================================================================
//	Install a listener and accept any connections on it. Call
//	HandleConnection( ) for each one. Subclasses should override
//	HandleConnection( ) and be sure to delete the CTCPStream they are
//	passed.

void CListenerThread::RunThread( )
{
	long err = B_NO_ERROR;
	
	err = mListener->Bind( mInterface, mPort );
	RETURN_ON_ERR( err );
	err = mListener->Listen( 4 );
	RETURN_ON_ERR( err );

	while ( 1 )
	{
		CTCPStream* newClient = mListener->Accept( );
		if ( newClient )
			HandleConnection( newClient );
		else
		{
			ReportError( mListener->Error( ), "Accept failed." );
			break;
		}
	}
	
clean_up:
	if ( err )
		ReportError( err, "A socket listener couldn't be installed." );
}


//=========================================================================
//	* CListenerThread::ThreadFinished
//=========================================================================
//	We will be called whenever the thread ends execution by returning from
//	RunThread() or when the Kill( ) method is called. We make an effort to
//	close our socket, but any other resources we may have allocated--
//	including the sockets of accepted connections--will be leaked if we are
//	Kill( )'d. 

void CListenerThread::ThreadFinished( )
{
	delete mListener;
}
